@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 # Copyright (c) 2012 - 2015 Lars Hupfeldt Nielsen, Hupfeldt IT
 # All rights reserved. This work is under a BSD license, see LICENSE.TXT.
@@ -15,7 +15,6 @@ if __package__ is None:
     __package__ = "jenkinsflow.cli"
 
 
-from .set_build_result import set_build_result
 from .set_build_description import set_build_description
 
 
@@ -25,8 +24,7 @@ def cli():
     pass
 
 
-cli.add_command(set_build_result)
-cli.add_command(set_build_description)
+cli.add_command(set_build_description, name="set_build_description")
 
 
 if __name__ == "__main__":
