@@ -6,15 +6,19 @@ from __future__ import print_function
 import sys, os, tempfile, time
 from collections import OrderedDict
 
-from .api_base import BuildResult, Progress, UnknownJobException, ApiInvocationMixin
+from .api_base import BuildResult, Progress, AuthError, ClientError, UnknownJobException, ApiInvocationMixin
 from .speed import Speed
 from .rest_api_wrapper import ResourceNotFound, RequestsRestApi
 
 
 major_version = sys.version_info.major
 
-jenkins_cli_jar = 'jenkins-cli.jar'
-hudson_cli_jar = 'hudson-cli.jar'
+
+if major_version < 3:
+    from .rest_api_wrapper import ConnectionError
+else:
+    import urllib.parse
+
 
 _superseded = -1
 _dequeued = -2
@@ -40,10 +44,12 @@ class Jenkins(Speed):
             which only has access to the jobs in the flow that it is executing. That way the job list will be filtered serverside.
         username (str): Name of user authorized to execute all jobs in flow.
         password (str): Password of user.
-        invocation_class (class): Defaults to `Invocation`.
+        invocation_class (class): Defaults to `Invocation`. You can subclass that to provide your own class.
+        csrf (bool): Will attempt to get (and use) a CSRF protection crumb from Jenkins. A 404 - ResourceNotFound error is silently
+            ignored as this indicates that csrf protection is not enabled on Jenkins.
     """
 
-    def __init__(self, direct_uri, job_prefix_filter=None, username=None, password=None, invocation_class=None, rest_access_provider=RequestsRestApi):
+    def __init__(self, direct_uri, job_prefix_filter=None, username=None, password=None, invocation_class=None, rest_access_provider=RequestsRestApi, csrf=True):
         if username or password:
             if not (username and password):
                 raise Exception("You must specify both username and password or neither")
@@ -58,15 +64,47 @@ class Jenkins(Speed):
         self.jobs = None
         self.queue_items = {}
         self.is_jenkins = None
+        self.csrf = csrf
+        self._crumb = None
 
-    def get_content(self, url, **params):
-        return self.rest_api.get_content(url, **params)
+    def _get_fresh_crumb(self):
+        """Get the CSRF crumb to be put on subsequent requests"""
+        try:
+            crumb = self.rest_api.get_content('/crumbIssuer/api/xml', xpath='concat(//crumbRequestField,":",//crumb)').split(b':')
+            self._crumb = {crumb[0]: crumb[1]}
+            return self._crumb
+        except ResourceNotFound:
+            self.csrf = False
 
     def get_json(self, url="", **params):
-        return self.rest_api.get_json(url, **params)
+        # Sometimes (but rarely) Jenkins will Abort the connection when jobs are being aborted!
+        for ii in (1, 2, 3):
+            try:
+                return self.rest_api.get_json(url, **params)
+            except ConnectionError as ex:
+                if ii == 3:
+                    raise
+                print("WARNING: Retrying failed 'poll':", ex)
+                time.sleep(0.1)
 
     def post(self, url, payload=None, headers=None, **params):
-        return self.rest_api.post(url, payload, headers, **params)
+        for crumb_attempt in (0, 1):
+            if self._crumb:
+                if headers:
+                    headers = headers.copy()
+                    headers.update(self._crumb)
+                else:
+                    headers = self._crumb
+
+            try:
+                return self.rest_api.post(url, payload, headers, **params)
+            except ClientError as ex:
+                if crumb_attempt:
+                    raise
+                if self.csrf:
+                    if self._crumb:
+                        print("INFO: getting new crumb:", ex)
+                    self._get_fresh_crumb()
 
     def headers(self):
         return self.rest_api.headers()
@@ -95,8 +133,10 @@ class Jenkins(Speed):
                     if head_response.get("X-Hudson"):
                         self.is_jenkins = False
                         break
+                except AuthError:
+                    raise
                 except Exception as ex:
-                    head_response = "HEAD request to " + repr(self.direct_uri) + " failed:" + str(ex)
+                    head_response = "HEAD request failed: " + str(ex)
                 time.sleep(0.1)
             else:
                 raise Exception("Not connected to Jenkins or Hudson (expected X-Jenkins or X-Hudson header, got: " + repr(head_response))
@@ -155,7 +195,9 @@ class Jenkins(Speed):
             raise UnknownJobException(self._public_job_url(name))
 
     def create_job(self, job_name, config_xml):
-        self.post('/createItem', name=job_name, headers={'Content-Type': 'application/xml header'}, payload=config_xml)
+        self.post('/createItem', name=job_name,
+                  headers={'Content-Type': 'application/xml header;charset=utf-8'},
+                  payload=config_xml if major_version < 3 else config_xml.encode('utf-8'))
 
     def delete_job(self, job_name):
         try:
@@ -164,18 +206,24 @@ class Jenkins(Speed):
             # TODO: Check error
             raise UnknownJobException(self._public_job_url(job_name), ex)
 
-    def set_build_description(self, job_name, build_number, description, replace=False, separator='\n'):
+    def set_build_description(self, description, replace=False, separator='\n', job_name=None, build_number=None):
         """Utility to set/append build description
 
         Args:
-            job_name (str):     Name of the Jenkins job
-            build_number (int): The build number for which to set the description
             description (str):  The description to set on the build
             replace (bool):     If True, replace existing description, if any, instead of appending to it
             separator (str):    A separator to insert between any existing description and the new :py:obj:`description` if :py:obj:`replace` is False.
+            job_name (str):     Name of the Jenkins job
+            build_number (int): The build number for which to set the description
         """
         self.poll()
 
+        if job_name is None:
+            job_name = os.environ['JOB_NAME']
+
+        if build_number is None:
+            build_number = int(os.environ['BUILD_NUMBER'])
+
         job_path = "/job/" + job_name
         build_url = job_path + '/' + str(build_number)
         try:
@@ -189,74 +237,6 @@ class Jenkins(Speed):
         except ResourceNotFound as ex:
             raise Exception("Build not found " + repr(build_url), ex)
 
-    def _download_cli(self, cli_jar):
-        public_uri = self.public_uri.rstrip('/') + '/'
-        direct_uri = self.direct_uri.rstrip('/') + '/'
-
-        path = 'jnlpJars/' + cli_jar
-        public_cli_url = public_uri + path
-        if direct_uri != public_uri:
-            download_cli_url = direct_uri + path
-            print("INFO: Downloading cli: '{public_cli_url}' (using direct url: '{direct_cli_url}')".format(
-                public_cli_url=public_cli_url, direct_cli_url=download_cli_url))
-        else:
-            download_cli_url = public_cli_url
-            print("INFO: Downloading cli: '{download_cli_url}'".format(download_cli_url=download_cli_url))
-
-        with open(cli_jar, 'w' if major_version < 3 else 'w+b') as ff:
-            ff.write(self.get_content('/' + path))
-        print("INFO: Download finished:", repr(cli_jar))
-
-    def set_build_result(self, result, java='java', cli_call=False):
-        """Change the result of a Jenkins job.
-
-        Note: set_build_result can only be done from within the job, not after the job has finished.
-        Note: Only available if URL is set in `Jenkins <http://jenkins-ci.org/>`_ system configuration.
-
-        This command uses the Jenkins `cli` to change the result. It requires a java executable to run the cli.
-        Note: In some versions of Jenkins the `cli` is broken, it has no manifest file! This is the case for
-        e.g. 1.625.1, the version installed on Fedora 23 using `dnf` at the time of Fedora 23 release.
-
-        Args:
-            result (str): The result to set. Should probably be 'unstable'
-            java (str): Alternative `java` executable. Use this if you don't wish to use the java in the PATH.
-        """
-
-        print("INFO: Setting job result to", repr(result))
-        cli_jar = jenkins_cli_jar if self.is_jenkins else hudson_cli_jar
-
-        if major_version < 3:
-            import subprocess32 as subprocess
-        else:
-            import subprocess
-
-        def set_res():
-            command = [java, '-jar', cli_jar, '-s', self.direct_uri, 'set-build-result', result]
-            if self.username:
-                fname = None
-                try:
-                    fhandle, fname = tempfile.mkstemp()
-                    fhandle = os.fdopen(fhandle, 'w')
-                    fhandle.write(self.password)
-                    fhandle.close()
-                    subprocess.check_call(command + ['--username', self.username, '--password-file', fname])
-                finally:
-                    try:
-                        os.remove(fname)
-                        fhandle.close()
-                    except IOError:  # pragma: no cover
-                        pass
-            else:
-                subprocess.check_call(command)
-
-        try:
-            # If cli_jar is already present attempt to use it
-            set_res()
-        except subprocess.CalledProcessError:
-            # We failed for some reason, try again with updated cli_jar
-            self._download_cli(cli_jar)
-            set_res()
-
 
 class ApiJob(object):
     def __init__(self, jenkins, dct, name):
@@ -268,6 +248,8 @@ class ApiJob(object):
         actions = self.dct.get('actions') or []
         self._path = "/job/" + self.name
         for action in actions:
+            if action is None:
+                continue
             if action.get('parameterDefinitions'):
                 self._build_trigger_path = self._path + "/buildWithParameters"
                 break
@@ -290,7 +272,14 @@ class ApiJob(object):
         except ResourceNotFound as ex:
             raise UnknownJobException(self.jenkins._public_job_url(self.name), ex)  # pylint: disable=protected-access
 
-        location = response.headers['location'][len(self.jenkins.direct_uri):-1]
+        if major_version >= 3:
+            # Make location relative
+            parts = urllib.parse.urlsplit(response.headers['location'])
+            location = urllib.parse.urlunsplit(['', ''] + list(parts[2:]))
+        else:
+            #  This hack is not correct because Jenkins may shange scheme from https to http :(
+            location = response.headers['location'][len(self.jenkins.direct_uri):]
+
         old_inv = self._invocations.get(location)
         if old_inv:
             old_inv.build_number = _superseded
@@ -378,7 +367,21 @@ class ApiJob(object):
                     pass
 
     def update_config(self, config_xml):
-        self.jenkins.post("/job/" + self.name + "/config.xml", payload=config_xml)
+        self.jenkins.post("/job/" + self.name + "/config.xml",
+                          headers={'Content-Type': 'application/xml header;charset=utf-8'},
+                          payload=config_xml if major_version < 3 else config_xml.encode('utf-8'))
+
+    def disable(self):
+        try:
+            self.jenkins.post(self._path + '/disable')
+        except ResourceNotFound as ex:
+            raise UnknownJobException(self.jenkins._public_job_url(self.name), ex)  # pylint: disable=protected-access
+
+    def enable(self):
+        try:
+            self.jenkins.post(self._path + '/enable')
+        except ResourceNotFound as ex:
+            raise UnknownJobException(self.jenkins._public_job_url(self.name), ex)  # pylint: disable=protected-access
 
     def __repr__(self):
         return str(dict(name=self.name, dct=self.dct))
